@@ -13,6 +13,7 @@ require 'gitlab/experiment/callbacks'
 require 'gitlab/experiment/rollout'
 require 'gitlab/experiment/configuration'
 require 'gitlab/experiment/cookies'
+require 'gitlab/experiment/force_assignment'
 require 'gitlab/experiment/context'
 require 'gitlab/experiment/dsl'
 require 'gitlab/experiment/middleware'
@@ -26,6 +27,7 @@ module Gitlab
     include BaseInterface
     include Cache
     include Callbacks
+    include ForceAssignment
     include Nestable
 
     class << self
@@ -107,6 +109,8 @@ module Gitlab
     end
 
     def assigned(value = nil)
+      # Skip force assignment if a variant was already set (e.g., via constructor or explicit #assigned call).
+      value ||= forced_variant_name unless @_assigned_variant_name
       @_assigned_variant_name = cache_variant(value) if value.present?
       return Variant.new(name: @_assigned_variant_name || :unresolved) if @_assigned_variant_name || @_resolving_variant
 
@@ -158,7 +162,11 @@ module Gitlab
     def excluded?
       return @_excluded if defined?(@_excluded)
 
-      @_excluded = !run_callbacks(exclusion_callback_chain) { :not_excluded }
+      @_excluded = !run_callbacks(exclusion_callback_chain) { :not_excluded } || only_assigned?
+    end
+
+    def only_assigned?
+      !!context.only_assigned && find_variant.blank?
     end
 
     def should_track?
