@@ -44,6 +44,15 @@ module Gitlab
         "#{experiment.name}_id"
       end
 
+      # Allow forced variant assignment via query parameter and cookie.
+      #
+      # When enabled, experiments will check for a `glex_force` query
+      # parameter and a `_glex_force` cookie to override variant assignment.
+      # This is intended for QA/UAT testing in staging and production.
+      #
+      # Defaults to false (disabled).
+      @allow_forced_assignment = false
+
       # Mark experiment cookies as secure (HTTPS only).
       #
       # When set to true, cookies will have the secure flag set, meaning they
@@ -187,6 +196,7 @@ module Gitlab
           :cookie_domain,
           :cookie_name,
           :secure_cookie,
+          :allow_forced_assignment,
           :context_key_secret,
           :context_key_bit_length,
           :mount_at,
